Class: Fog::AWS::Compute::Server

Inherits:
Model
  • Object
show all
Extended by:
Deprecation
Defined in:
lib/fog/compute/models/aws/server.rb

Instance Attribute Summary collapse

Attributes inherited from Model

#collection, #connection

Instance Method Summary collapse

Methods included from Deprecation

deprecate, self_deprecate

Methods inherited from Model

#inspect, #reload, #to_json, #wait_for

Methods included from Fog::Attributes::ClassMethods

#_load, #aliases, #attribute, #attributes, #identity, #ignore_attributes, #ignored_attributes

Methods included from Fog::Attributes::InstanceMethods

#_dump, #attributes, #identity, #identity=, #merge_attributes, #new_record?, #requires

Constructor Details

#initialize(attributes = {}) ⇒ Server

Returns a new instance of Server.



44
45
46
47
48
# File 'lib/fog/compute/models/aws/server.rb', line 44

def initialize(attributes={})
  self.groups ||= ["default"] unless attributes[:subnet_id]
  self.flavor_id ||= 'm1.small'
  super
end

Instance Attribute Details

#architecture ⇒ Object

Returns the value of attribute architecture.



13
14
15
# File 'lib/fog/compute/models/aws/server.rb', line 13

def architecture
  @architecture
end

#instance_initiated_shutdown_behavior ⇒ Object

Returns the value of attribute instance_initiated_shutdown_behavior.



22
23
24
# File 'lib/fog/compute/models/aws/server.rb', line 22

def instance_initiated_shutdown_behavior
  @instance_initiated_shutdown_behavior
end

#password ⇒ Object

Returns the value of attribute password.



41
42
43
# File 'lib/fog/compute/models/aws/server.rb', line 41

def password
  @password
end

#private_key ⇒ Object



97
98
99
# File 'lib/fog/compute/models/aws/server.rb', line 97

def private_key
  @private_key ||= private_key_path && File.read(private_key_path)
end

#private_key_path ⇒ Object



92
93
94
95
# File 'lib/fog/compute/models/aws/server.rb', line 92

def private_key_path
  @private_key_path ||= Fog.credentials[:private_key_path]
  @private_key_path &&= File.expand_path(@private_key_path)
end

#public_key ⇒ Object



106
107
108
# File 'lib/fog/compute/models/aws/server.rb', line 106

def public_key
  @public_key ||= public_key_path && File.read(public_key_path)
end

#public_key_path ⇒ Object



101
102
103
104
# File 'lib/fog/compute/models/aws/server.rb', line 101

def public_key_path
  @public_key_path ||= Fog.credentials[:public_key_path]
  @public_key_path &&= File.expand_path(@public_key_path)
end

#username ⇒ Object



209
210
211
# File 'lib/fog/compute/models/aws/server.rb', line 209

def username
  @username ||= 'root'
end

Instance Method Details

#addresses ⇒ Object



50
51
52
53
54
# File 'lib/fog/compute/models/aws/server.rb', line 50

def addresses
  requires :id

  connection.addresses(:server => self)
end

#console_output ⇒ Object



56
57
58
59
60
# File 'lib/fog/compute/models/aws/server.rb', line 56

def console_output
  requires :id

  connection.get_console_output(id)
end

#destroy ⇒ Object



62
63
64
65
66
67
# File 'lib/fog/compute/models/aws/server.rb', line 62

def destroy
  requires :id

  connection.terminate_instances(id)
  true
end

#flavor ⇒ Object



78
79
80
# File 'lib/fog/compute/models/aws/server.rb', line 78

def flavor
  @flavor ||= connection.flavors.all.detect {|flavor| flavor.id == flavor_id}
end

#flavor=(new_flavor) ⇒ Object



74
75
76
# File 'lib/fog/compute/models/aws/server.rb', line 74

def flavor=(new_flavor)
  @flavor = new_flavor
end

#flavor_id ⇒ Object



70
71
72
# File 'lib/fog/compute/models/aws/server.rb', line 70

def flavor_id
  @flavor && @flavor.id || attributes[:flavor_id]
end

#key_pair ⇒ Object



82
83
84
85
86
# File 'lib/fog/compute/models/aws/server.rb', line 82

def key_pair
  requires :key_name

  connection.keypairs.all(key_name).first
end

#key_pair=(new_keypair) ⇒ Object



88
89
90
# File 'lib/fog/compute/models/aws/server.rb', line 88

def key_pair=(new_keypair)
  self.key_name = new_keypair && new_keypair.name
end

#ready? ⇒ Boolean

Returns:

  • (Boolean)


110
111
112
# File 'lib/fog/compute/models/aws/server.rb', line 110

def ready?
  state == 'running'
end

#reboot ⇒ Object



114
115
116
117
118
# File 'lib/fog/compute/models/aws/server.rb', line 114

def reboot
  requires :id
  connection.reboot_instances(id)
  true
end

#save ⇒ Object

Raises:



120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
# File 'lib/fog/compute/models/aws/server.rb', line 120

def save
  raise Fog::Errors::Error.new('Resaving an existing object may create a duplicate') if identity
  requires :image_id

  options = {
    'BlockDeviceMapping'          => block_device_mapping,
    'ClientToken'                 => client_token,
    'InstanceInitiatedShutdownBehavior' => instance_initiated_shutdown_behavior,
    'InstanceType'                => flavor_id,
    'KernelId'                    => kernel_id,
    'KeyName'                     => key_name,
    'Monitoring.Enabled'          => monitoring,
    'Placement.AvailabilityZone'  => availability_zone,
    'RamdiskId'                   => ramdisk_id,
    'SecurityGroup'               => groups,
    'SubnetId'                    => subnet_id,
    'UserData'                    => user_data
  }
  options.delete_if {|key, value| value.nil?}

  # If subnet is defined we are working on a virtual private cloud.
  # subnet & security group cannot co-exist. I wish VPC just ignored
  # the security group parameter instead, it would be much easier!
  if subnet_id
    options.delete('SecurityGroup')
  else
    options.delete('SubnetId')
  end

  data = connection.run_instances(image_id, 1, 1, options)
  merge_attributes(data.body['instancesSet'].first)
  true
end

#scp(local_path, remote_path) ⇒ Object



189
190
191
192
193
194
195
# File 'lib/fog/compute/models/aws/server.rb', line 189

def scp(local_path, remote_path)
  requires :public_ip_address, :username

  options = {}
  options[:key_data] = [private_key] if private_key
  Fog::SCP.new(public_ip_address, username, options).upload(local_path, remote_path)
end

#setup(credentials = {}) ⇒ Object



154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
# File 'lib/fog/compute/models/aws/server.rb', line 154

def setup(credentials = {})
  requires :identity, :ip_address, :username
  require 'json'

  commands = [
    %{mkdir .ssh},
    %{passwd -l #{username}},
    %{echo "#{attributes.to_json}" >> ~/attributes.json}
  ]
  if public_key
    commands << %{echo "#{public_key}" >> ~/.ssh/authorized_keys}
  end
  # allow some retries over the first 120 seconds because aws is weird
  Timeout::timeout(120) do
    begin
      Timeout::timeout(4) do
        Fog::SSH.new(ip_address, username, credentials).run(commands)
      end
    rescue Net::SSH::AuthenticationFailed, Timeout::Error
      retry
    end
  end
rescue Errno::ECONNREFUSED => e
  sleep(1)
  retry
end

#ssh(commands) ⇒ Object



181
182
183
184
185
186
187
# File 'lib/fog/compute/models/aws/server.rb', line 181

def ssh(commands)
  requires :identity, :public_ip_address, :username

  options = {}
  options[:key_data] = [private_key] if private_key
  Fog::SSH.new(public_ip_address, username, options).run(commands)
end

#start ⇒ Object



197
198
199
200
201
# File 'lib/fog/compute/models/aws/server.rb', line 197

def start
  requires :id
  connection.start_instances(id)
  true
end

#stop ⇒ Object



203
204
205
206
207
# File 'lib/fog/compute/models/aws/server.rb', line 203

def stop
  requires :id
  connection.stop_instances(id)
  true
end

#volumes ⇒ Object



213
214
215
216
# File 'lib/fog/compute/models/aws/server.rb', line 213

def volumes
  requires :id
  connection.volumes(:server => self)
end